//+------------------------------------------------------------------+
//|                       Indicator: Signal_Stochastics_Strategy.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " "
#property link      " "
#property version   "1.00"
#property description "Trade extreme levels of overbought and oversold"

#include <stdlib.mqh>
#include <stderror.mqh>

//--- indicator settings
#property indicator_chart_window
#property indicator_buffers 2

#property indicator_type1 DRAW_ARROW
#property indicator_width1 2
#property indicator_color1 0xFFAA00
#property indicator_label1 "CALL opportunity"

#property indicator_type2 DRAW_ARROW
#property indicator_width2 2
#property indicator_color2 0x006AFF
#property indicator_label2 "PUT opportunity"

//--- indicator buffers
double Buffer1[];
double Buffer2[];

double myPoint; //initialized in OnInit

void myAlert(string type, string message)
  {
   if(type == "print")
      Print(message);
   else if(type == "error")
     {
      Print(type+" | Signal_Stochastics_Strategy @ "+Symbol()+","+Period()+" | "+message);
     }
   else if(type == "order")
     {
     }
   else if(type == "modify")
     {
     }
  }

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {   
   IndicatorBuffers(2);
   SetIndexBuffer(0, Buffer1);
   SetIndexEmptyValue(0, 0);
   SetIndexArrow(0, 241);
   SetIndexBuffer(1, Buffer2);
   SetIndexEmptyValue(1, 0);
   SetIndexArrow(1, 242);
   //initialize myPoint
   myPoint = Point();
   if(Digits() == 5 || Digits() == 3)
     {
      myPoint *= 10;
     }
   return(INIT_SUCCEEDED);
  }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
  {
   int limit = rates_total - prev_calculated;
   //--- counting from 0 to rates_total
   ArraySetAsSeries(Buffer1, true);
   ArraySetAsSeries(Buffer2, true);
   //--- initial zero
   if(prev_calculated < 1)
     {
      ArrayInitialize(Buffer1, 0);
      ArrayInitialize(Buffer2, 0);
     }
   else
      limit++;
   
   //--- main loop
   for(int i = limit-1; i >= 0; i--)
     {
      if (i >= MathMin(5000-1, rates_total-1-50)) continue; //omit some old rates to prevent "Array out of range" or slow calculation   
      //Indicator Buffer 1
      if(iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_MAIN, i) > iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_SIGNAL, i)
      && iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_MAIN, i+1) < iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_SIGNAL, i+1) //Stochastic Oscillator crosses above Stochastic Oscillator
      && iStochastic(NULL, PERIOD_CURRENT, 14, 3, 3, MODE_SMA, 0, MODE_MAIN, i) < 20 //Stochastic Oscillator < fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 14, 3, 3, MODE_SMA, 0, MODE_SIGNAL, i) < 20 //Stochastic Oscillator < fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_MAIN, i) < 20 //Stochastic Oscillator < fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_SIGNAL, i) < 20 //Stochastic Oscillator < fixed value
      )
        {
         Buffer1[i] = Low[i]; //Set indicator value at Candlestick Low
        }
      //Indicator Buffer 2
      if(iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_MAIN, i) < iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_SIGNAL, i)
      && iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_MAIN, i+1) > iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_SIGNAL, i+1) //Stochastic Oscillator crosses below Stochastic Oscillator
      && iStochastic(NULL, PERIOD_CURRENT, 14, 3, 3, MODE_SMA, 0, MODE_MAIN, i) > 80 //Stochastic Oscillator > fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 14, 3, 3, MODE_SMA, 0, MODE_SIGNAL, i) > 80 //Stochastic Oscillator > fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_MAIN, i) > 80 //Stochastic Oscillator > fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 21, 9, 9, MODE_SMA, 0, MODE_SIGNAL, i) > 80 //Stochastic Oscillator > fixed value
      )
        {
         Buffer2[i] = High[i]; //Set indicator value at Candlestick High
        }
     }
   return(rates_total);
  }
//+------------------------------------------------------------------+